class MitoSiteApp {
  constructor(containerSelector = ".scrollWebcam") {
    this.containers = document.querySelectorAll(containerSelector)
    this.instances = new Map()
    this.model = null
    this.isDetecting = false
    this.minimaps = [] // Array to store minimap objects
  }

  async init() {
    // Load COCO-SSD model using the official library
    try {
      // Load the model directly from the CDN
      this.model = await cocoSsd.load()
      console.log("TensorFlow model loaded successfully")
    } catch (err) {
      console.error("Error loading TensorFlow model:", err)
    }
    this.containers.forEach((container) => this.setupContainer(container))
    this.updateZoomDisplay()
  }

  // Modified toggleHelpCommand method
  toggleHelpCommand(container) {
    let helpModal = container.querySelector(".help-modal")

    if (helpModal) {
      // If modal exists, toggle its visibility
      const isVisible = helpModal.style.display !== "none"
      helpModal.style.display = isVisible ? "none" : "block"
      return
    }

    // Create help modal
    helpModal = document.createElement("div")
    helpModal.className = "help-modal"
    helpModal.style.cssText = `
    position: fixed;
    top: 0;
    right: 0;
    width: 300px;
    height: 100vh;
    background: rgba(0, 0, 0, 0.85);
    color: white;
    padding: 20px;
    overflow-y: auto;
    z-index: 1000;
    box-shadow: -2px 0 10px rgba(0, 0, 0, 0.3);
  `

    // Create heading
    const heading = document.createElement("h2")
    heading.textContent = "Keyboard Shortcuts"
    heading.style.marginBottom = "20px"
    helpModal.appendChild(heading)

    // Parse shortcuts and create clickable elements
    const shortcuts = this.keyboardShortcuts
      .split("\n")
      .slice(1) // Skip header row
      .map((line) => {
        const [key, command, category] = line.split(" ")
        return { key, command, category }
      })

    // Group shortcuts by category
    const categorizedShortcuts = shortcuts.reduce(
      (acc, { key, command, category }) => {
        if (!acc[category]) {
          acc[category] = []
        }
        acc[category].push({ key, command })
        return acc
      },
      {},
    )

    // Create sections for each category
    Object.entries(categorizedShortcuts).forEach(([category, shortcuts]) => {
      const categorySection = document.createElement("div")
      categorySection.style.marginBottom = "20px"

      const categoryHeading = document.createElement("h3")
      categoryHeading.textContent = category
      categoryHeading.style.marginBottom = "10px"
      categorySection.appendChild(categoryHeading)

      shortcuts.forEach(({ key, command }) => {
        const shortcutElement = document.createElement("div")
        shortcutElement.style.cssText = `
        display: flex;
        align-items: center;
        margin-bottom: 10px;
        cursor: pointer;
        padding: 5px;
        border-radius: 4px;
      `

        // Add hover effect
        shortcutElement.addEventListener("mouseenter", () => {
          shortcutElement.style.background = "rgba(255, 255, 255, 0.1)"
        })
        shortcutElement.addEventListener("mouseleave", () => {
          shortcutElement.style.background = "none"
        })

        const keyElement = document.createElement("kbd")
        keyElement.textContent = key
        keyElement.style.cssText = `
        background: rgba(255, 255, 255, 0.2);
        padding: 2px 6px;
        border-radius: 3px;
        margin-right: 10px;
        min-width: 20px;
        text-align: center;
      `

        const commandName = command.replace("Command", "")
        const displayName = commandName
          .split(/(?=[A-Z])/)
          .map((word) => word.charAt(0).toUpperCase() + word.slice(1))
          .join(" ")

        const descElement = document.createElement("span")
        descElement.textContent = displayName

        shortcutElement.appendChild(keyElement)
        shortcutElement.appendChild(descElement)

        // Add click handler to trigger the command
        shortcutElement.addEventListener("click", () => {
          if (typeof this[command] === "function") {
            this[command](container)
          }
        })

        categorySection.appendChild(shortcutElement)
      })

      helpModal.appendChild(categorySection)
    })

    // Add close button
    const closeButton = document.createElement("button")
    closeButton.textContent = "×"
    closeButton.style.cssText = `
    position: absolute;
    top: 10px;
    right: 10px;
    background: none;
    border: none;
    color: white;
    font-size: 24px;
    cursor: pointer;
    padding: 5px;
  `
    closeButton.addEventListener("click", () => {
      helpModal.style.display = "none"
    })
    helpModal.appendChild(closeButton)

    // Add to document body instead of container
    document.body.appendChild(helpModal)
  }

  keyboardShortcuts = `key command category
d toggleDetectionCommand Detection
t drawRandomBoxesCommand Detection
h flipHorizontalCommand Camera
v flipVerticalCommand Camera
c takeSnapshotCommand Camera
z changeZoomLevelDisplayCommand Overlay
m createMinimapCommand Overlay
? toggleHelpCommand Overlay`

  createMinimapCommand(container) {
    const data = this.instances.get(container)

    // Create a new minimap object
    const minimap = {
      canvas: document.createElement("canvas"),
      zoomLevel: this.currentZoomLevel,
      imageData: null,
    }

    // Set up canvas properties
    minimap.canvas.className = "minimap"
    minimap.canvas.width = this.side
    minimap.canvas.height = this.side

    // Calculate position based on number of existing minimaps
    const offset = this.minimaps.length * 220 // 200px width + 20px spacing
    minimap.canvas.style.cssText = `
      position: fixed;
      right: ${10 + offset}px;
      top: 10px;
      width: 200px;
      height: 200px;
      border: 2px solid white;
      background: black;
      display: block;
    `

    // Add zoom level label
    const label = document.createElement("div")
    label.style.cssText = `
      position: fixed;
      right: ${10 + offset}px;
      top: 220px;
      width: 200px;
      text-align: center;
      color: white;
      font-size: 14px;
      background: rgba(0, 0, 0, 0.7);
      padding: 4px;
      border-radius: 4px;
    `
    label.textContent = `${this.currentZoomLevel} View`
    minimap.label = label

    // Draw initial image
    const ctx = minimap.canvas.getContext("2d")
    ctx.drawImage(data.video, 0, 0, minimap.canvas.width, minimap.canvas.height)
    minimap.imageData = ctx.getImageData(
      0,
      0,
      minimap.canvas.width,
      minimap.canvas.height,
    )

    // Add close button
    const closeButton = document.createElement("button")
    closeButton.style.cssText = `
      position: absolute;
      top: 5px;
      right: 5px;
      background: rgba(255, 255, 255, 0.3);
      border: none;
      color: white;
      width: 20px;
      height: 20px;
      border-radius: 50%;
      cursor: pointer;
      display: flex;
      align-items: center;
      justify-content: center;
      font-size: 12px;
    `
    closeButton.textContent = "×"
    closeButton.onclick = (e) => {
      e.stopPropagation()
      this.removeMinimapByIndex(this.minimaps.indexOf(minimap))
    }
    minimap.canvas.appendChild(closeButton)

    // Add to DOM and store in minimaps array
    document.body.appendChild(minimap.canvas)
    document.body.appendChild(minimap.label)
    this.minimaps.push(minimap)

    this.updateAllMinimaps()
  }

  removeMinimapByIndex(index) {
    if (index >= 0 && index < this.minimaps.length) {
      const minimap = this.minimaps[index]
      minimap.canvas.remove()
      minimap.label.remove()
      this.minimaps.splice(index, 1)

      // Reposition remaining minimaps
      this.minimaps.forEach((map, idx) => {
        const offset = idx * 220
        map.canvas.style.right = `${10 + offset}px`
        map.label.style.right = `${10 + offset}px`
      })
    }
  }

  updateAllMinimaps() {
    this.minimaps.forEach((minimap) => {
      const ctx = minimap.canvas.getContext("2d")
      ctx.clearRect(0, 0, minimap.canvas.width, minimap.canvas.height)

      // Restore the original image
      ctx.putImageData(minimap.imageData, 0, 0)

      // Calculate box size based on zoom levels
      const zoomRatio =
        parseInt(minimap.zoomLevel) / parseInt(this.currentZoomLevel)
      const boxSize = this.side * zoomRatio

      // Draw red box centered in minimap
      ctx.strokeStyle = "red"
      ctx.lineWidth = 2
      const x = (minimap.canvas.width - boxSize) / 2
      const y = (minimap.canvas.height - boxSize) / 2
      ctx.strokeRect(x, y, boxSize, boxSize)
    })
  }

  setupKeyboardShortcuts(container) {
    // Parse the keyboard shortcuts string into a map
    const shortcuts = new Map(
      this.keyboardShortcuts
        .split("\n")
        .slice(1) // Skip header row
        .map((line) => {
          const [key, command] = line.split(" ")
          return [key, command]
        }),
    )

    // Add keyboard event listener
    document.addEventListener("keydown", (e) => {
      // Ignore if user is typing in an input field
      if (e.target.tagName.toLowerCase() === "input") return

      const command = shortcuts.get(e.key.toLowerCase())
      if (command && typeof this[command] === "function") {
        e.preventDefault()
        this[command](container)
      }
    })
  }

  toggleDetectionCommand(container) {
    this.isDetecting = !this.isDetecting
    if (this.isDetecting) {
      this.startDetection(container)
      console.log("Detection enabled")
    } else {
      // Clear all boxes when detection is disabled
      const data = this.instances.get(container)
      data.boxes = []
      this.clearCanvas(container)

      // Update cell counter
      const hud = container.querySelector(".scrollWebcamHud")
      const cellCounter = hud.querySelector(
        '.scrollWebcamHudElement[data-type="cellCount"]',
      )
      if (cellCounter) {
        cellCounter.textContent = "Cells Counted: 0"
      }

      console.log("Detection disabled")
    }
  }

  flipHorizontalCommand(container) {
    const data = this.instances.get(container)
    const { video } = data

    // Toggle the transform style
    const currentTransform = video.style.transform || ""
    if (currentTransform.includes("scaleX(-1)")) {
      video.style.transform = currentTransform.replace("scaleX(-1)", "")
    } else {
      video.style.transform = `${currentTransform} scaleX(-1)`
    }

    // Also flip the canvas to match
    const { canvas } = data
    if (currentTransform.includes("scaleX(-1)")) {
      canvas.style.transform = currentTransform.replace("scaleX(-1)", "")
    } else {
      canvas.style.transform = `${currentTransform} scaleX(-1)`
    }
  }

  flipVerticalCommand(container) {
    const data = this.instances.get(container)
    const { video } = data

    // Toggle the transform style
    const currentTransform = video.style.transform || ""
    if (currentTransform.includes("scaleY(-1)")) {
      video.style.transform = currentTransform.replace("scaleY(-1)", "")
    } else {
      video.style.transform = `${currentTransform} scaleY(-1)`
    }

    // Also flip the canvas to match
    const { canvas } = data
    if (currentTransform.includes("scaleY(-1)")) {
      canvas.style.transform = currentTransform.replace("scaleY(-1)", "")
    } else {
      canvas.style.transform = `${currentTransform} scaleY(-1)`
    }
  }

  takeSnapshotCommand(container) {
    const data = this.instances.get(container)
    const { video, canvas } = data

    // Create a temporary canvas to capture the full state
    const tempCanvas = document.createElement("canvas")
    tempCanvas.width = video.videoWidth
    tempCanvas.height = video.videoHeight
    const tempCtx = tempCanvas.getContext("2d")

    // 1. Draw the video frame
    tempCtx.drawImage(video, 0, 0)

    // 2. Draw the detection boxes from the canvas
    tempCtx.drawImage(canvas, 0, 0)

    // 3. Draw HUD elements
    const hud = container.querySelector(".scrollWebcamHud")
    if (hud) {
      // Get all HUD elements
      const hudElements = hud.querySelectorAll(".scrollWebcamHudElement")

      hudElements.forEach((element) => {
        // Get computed styles
        const style = window.getComputedStyle(element)
        const rect = element.getBoundingClientRect()
        const containerRect = container.getBoundingClientRect()

        // Calculate relative position within container
        const x = rect.left - containerRect.left
        const y = rect.top - containerRect.top

        // Set up text styling
        tempCtx.font = style.font || "30px Arial"
        tempCtx.fillStyle = style.color || "white"
        tempCtx.textBaseline = "top"

        // Apply text shadow if present
        if (style.textShadow && style.textShadow !== "none") {
          tempCtx.shadowColor = "black"
          tempCtx.shadowBlur = 2
          tempCtx.shadowOffsetX = 1
          tempCtx.shadowOffsetY = 1
        }

        // Draw the text
        tempCtx.fillText(
          element.textContent,
          (x / containerRect.width) * video.videoWidth,
          (y / containerRect.height) * video.videoHeight,
        )

        // Reset shadow
        tempCtx.shadowColor = "transparent"
        tempCtx.shadowBlur = 0
        tempCtx.shadowOffsetX = 0
        tempCtx.shadowOffsetY = 0
      })
    }

    // 4. Draw zoom level indicator
    const zoomDisplay = container.querySelector(".top-left")
    if (zoomDisplay) {
      const style = window.getComputedStyle(zoomDisplay)
      tempCtx.font = style.font || "16px Arial"
      tempCtx.fillStyle = style.color || "white"
      tempCtx.textBaseline = "top"
      tempCtx.fillText(zoomDisplay.textContent, 10, 10)
    }

    // Create thumbnail container if it doesn't exist
    let thumbContainer = container.querySelector(".snapshot-thumbnails")
    if (!thumbContainer) {
      thumbContainer = document.createElement("div")
      thumbContainer.className = "snapshot-thumbnails"
      thumbContainer.style.position = "fixed"
      thumbContainer.style.left = "10px"
      thumbContainer.style.top = "10px"
      thumbContainer.style.width = "120px"
      thumbContainer.style.zIndex = "1000"
      thumbContainer.style.maxHeight = "100vh"
      thumbContainer.style.overflowY = "auto"
      thumbContainer.style.padding = "10px"
      thumbContainer.style.background = "rgba(0, 0, 0, 0.5)"
      thumbContainer.style.borderRadius = "5px"
      container.appendChild(thumbContainer)
    }

    // Create and add thumbnail
    const thumbnail = document.createElement("div")
    thumbnail.style.marginBottom = "10px"
    thumbnail.style.cursor = "pointer"
    thumbnail.style.position = "relative"

    // Add timestamp to thumbnail
    const timestamp = document.createElement("div")
    timestamp.style.position = "absolute"
    timestamp.style.bottom = "0"
    timestamp.style.left = "0"
    timestamp.style.right = "0"
    timestamp.style.background = "rgba(0,0,0,0.5)"
    timestamp.style.color = "white"
    timestamp.style.fontSize = "10px"
    timestamp.style.padding = "2px"
    timestamp.textContent = new Date().toLocaleTimeString()

    const img = document.createElement("img")
    img.src = tempCanvas.toDataURL("image/png")
    img.style.width = "100%"
    img.style.border = "2px solid white"

    thumbnail.appendChild(img)
    thumbnail.appendChild(timestamp)
    thumbContainer.appendChild(thumbnail)

    // Add click handler to open full size in new window
    thumbnail.addEventListener("click", () => {
      const win = window.open()
      win.document.write(`
            <html>
                <head>
                    <title>Snapshot - ${new Date().toLocaleString()}</title>
                    <style>
                        body { margin: 0; background: #000; display: flex; justify-content: center; align-items: center; min-height: 100vh; }
                        img { max-width: 100%; max-height: 100vh; object-fit: contain; }
                    </style>
                </head>
                <body>
                    <img src="${img.src}" alt="Snapshot">
                </body>
            </html>
        `)
    })
  }

  changeZoomLevelDisplayCommand(container) {
    // Get next zoom level
    const currentIndex = this.zoomLevels.indexOf(this.currentZoomLevel)
    const nextIndex = (currentIndex + 1) % this.zoomLevels.length
    this.currentZoomLevel = this.zoomLevels[nextIndex]
    this.updateZoomDisplay()
    this.updateAllMinimaps()
  }

  updateZoomDisplay() {
    // Update or create zoom display element
    let zoomDisplay = document.querySelector(".top-left")
    // Update display text
    zoomDisplay.textContent = `${this.microscopeName} ${this.currentZoomLevel}`
  }

  microscopeName = "Swift SW200DL"
  zoomLevels = ["40x", "100x", "400x"]
  currentZoomLevel = "40x"

  async setupContainer(container) {
    const instanceData = {
      video: container.querySelector("video"),
      select: document.querySelector(".scrollWebcamSelect"),
      currentStream: null,
      boxes: [],
      isDrawing: false,
      startX: 0,
      startY: 0,
    }

    this.instances.set(container, instanceData)

    await this.setupWebcam(container)
    this.setupCanvas(container)
    this.setupHUD(container)

    // Start detection loop once everything is set up
    this.startDetection(container)
  }

  async startDetection(container) {
    const data = this.instances.get(container)
    const { video } = data

    if (!this.model) {
      console.error("TensorFlow model not loaded")
      return
    }

    // Wait for video to be ready
    if (video.readyState === 0) {
      video.addEventListener("loadeddata", () => this.startDetection(container))
      return
    }

    const detectFrame = async () => {
      if (!this.isDetecting) return

      // Check if video is ready and has valid dimensions
      if (
        video.readyState !== 4 ||
        video.videoWidth === 0 ||
        video.videoHeight === 0
      ) {
        requestAnimationFrame(() => detectFrame())
        return
      }

      try {
        // Make sure video dimensions are set
        if (video.width !== video.videoWidth) {
          video.width = video.videoWidth
        }
        if (video.height !== video.videoHeight) {
          video.height = video.videoHeight
        }

        // Perform detection only if video is playing and visible
        if (!video.paused && !video.ended && video.style.display !== "none") {
          const predictions = await this.model.detect(video)

          // Clear previous boxes
          this.clearCanvas(container)

          // Draw AI predictions in a different color without adding to boxes
          predictions.forEach((prediction) => {
            if (prediction.score > 0.5) {
              // Confidence threshold
              const [x, y, width, height] = prediction.bbox

              // Draw AI predictions in blue
              const { ctx } = data
              ctx.strokeStyle = "#0000ff"
              ctx.lineWidth = 2
              ctx.strokeRect(x, y, width, height)

              if (prediction.score !== null) {
                ctx.fillStyle = "#0000ff"
                ctx.font = "12px Arial"
                ctx.fillText(`${Math.round(prediction.score * 100)}%`, x, y - 5)
              }
            }
          })

          // Redraw manual boxes in green
          data.boxes.forEach((box) => {
            this.drawBox(container, box.x, box.y, box.width, box.height)
          })
        }
      } catch (err) {
        console.error("Detection error:", err)
      }

      // Request next frame
      requestAnimationFrame(() => detectFrame())
    }

    // Start detection loop
    this.isDetecting = true
    detectFrame()
  }

  drawBox(container, x, y, width, height, score = null) {
    const data = this.instances.get(container)
    const { ctx } = data

    ctx.strokeStyle = "#00ff00"
    ctx.lineWidth = 2
    ctx.strokeRect(x, y, width, height)

    if (score !== null) {
      ctx.fillStyle = "#00ff00"
      ctx.font = "12px Arial"
      ctx.fillText(`${Math.round(score * 100)}%`, x, y - 5)
    }
  }

  updateCellCount(container) {
    const data = this.instances.get(container)
    const hud = container.querySelector(".scrollWebcamHud")
    const cellCounter = hud.querySelector(
      '.scrollWebcamHudElement[data-type="cellCount"]',
    )
    if (cellCounter) {
      cellCounter.textContent = `Cells Detected: ${data.boxes.length}`
    }
  }

  async setupWebcam(container) {
    const data = this.instances.get(container)
    const { video, select } = data

    try {
      const devices = await navigator.mediaDevices.enumerateDevices()
      const videoDevices = devices.filter(
        (device) => device.kind === "videoinput",
      )

      // Add demo video option first
      const demoOption = document.createElement("option")
      demoOption.value = "demo"
      demoOption.text = "Demo Video"
      select.appendChild(demoOption)

      // Add webcam options
      videoDevices.forEach((device) => {
        const option = document.createElement("option")
        option.value = device.deviceId
        option.text = device.label || `Camera ${select.length}`
        select.appendChild(option)
      })

      select.style.display = "none"
      select.addEventListener("change", (e) => {
        if (e.target.value === "demo") {
          this.startDemoVideo(container)
        } else {
          this.startCamera(container, e.target.value)
        }
      })

      await this.startCamera(container)
    } catch (err) {
      console.error("Error getting camera list:", err)
      await this.startCamera(container)
    }
  }

  async startDemoVideo(container) {
    const data = this.instances.get(container)
    const { video, currentStream } = data

    // Stop any existing stream
    if (currentStream) {
      currentStream.getTracks().forEach((track) => track.stop())
      data.currentStream = null
    }

    // Set up video source
    video.src = "rbc.mp4" // Make sure this file exists in your project
    video.loop = true // Loop the video
    video.play().catch((err) => {
      console.error("Error playing demo video:", err)
      container.innerHTML = "Error: Could not play demo video"
    })
  }

  side = 700

  async startCamera(container, deviceId = null) {
    const { side } = this
    const data = this.instances.get(container)
    const { video, currentStream } = data

    if (currentStream) {
      currentStream.getTracks().forEach((track) => track.stop())
    }

    try {
      const constraints = {
        video: {
          width: parseInt(container.dataset.width) || side,
          height: parseInt(container.dataset.height) || side,
        },
      }

      if (deviceId) {
        constraints.video.deviceId = { exact: deviceId }
      }

      const stream = await navigator.mediaDevices.getUserMedia(constraints)
      data.currentStream = stream
      video.srcObject = stream

      // Wait for video to be ready before playing
      await new Promise((resolve) => {
        video.onloadedmetadata = () => {
          video.width = video.videoWidth
          video.height = video.videoHeight
          resolve()
        }
      })

      await video.play()
    } catch (err) {
      console.error("Error accessing webcam:", err)
      container.innerHTML = "Error: Could not access webcam"
    }
  }

  setupCanvas(container) {
    const data = this.instances.get(container)
    const { video } = data

    const canvas = document.createElement("canvas")
    canvas.className = "scrollWebcamCanvas"
    canvas.width = video.videoWidth || video.width
    canvas.height = video.videoHeight || video.height
    canvas.style.position = "absolute"
    canvas.style.top = "0"
    canvas.style.left = "0"
    canvas.style.zIndex = "1"
    canvas.style.cursor = "crosshair"
    canvas.style.width = "100%" // Make canvas scale with container
    canvas.style.height = "100%"

    container.insertBefore(canvas, container.firstChild.nextSibling)

    data.canvas = canvas
    data.ctx = canvas.getContext("2d")

    this.setupCanvasEventListeners(container)
    this.setupKeyboardShortcuts(container)
  }

  setupHUD(container) {
    const hud = container.querySelector(".scrollWebcamHud")
    if (hud) {
      // Create cell counter element if it doesn't exist
      let cellCounter = hud.querySelector(
        '.scrollWebcamHudElement[data-type="cellCount"]',
      )
      if (!cellCounter) {
        cellCounter = document.createElement("div")
        cellCounter.className = "scrollWebcamHudElement"
        cellCounter.dataset.type = "cellCount"
        cellCounter.style.position = "absolute"
        cellCounter.style.bottom = "10px"
        cellCounter.style.left = "10px"
        cellCounter.style.fontSize = "30px"
        cellCounter.style.textShadow = "1px 1px 2px black"
        hud.appendChild(cellCounter)
      }

      setInterval(() => {
        hud.querySelectorAll(".scrollWebcamHudElement").forEach((el) => {
          if (el.dataset.type === "time") {
            const now = new Date()
            const days = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"]
            const dayOfWeek = days[now.getDay()]
            el.textContent = `${dayOfWeek} ${now.toLocaleDateString()} ${now.toLocaleTimeString()}`
          } else if (el.dataset.type === "cellCount") {
            const data = this.instances.get(container)
            el.textContent = `Cells Counted: ${data.boxes.length}`
          }
        })
      }, 1000)
    }
  }

  setupCanvasEventListeners(container) {
    const data = this.instances.get(container)
    const { canvas } = data

    canvas.addEventListener("mousedown", (e) =>
      this.handleMouseDown(container, e),
    )
    canvas.addEventListener("mousemove", (e) =>
      this.handleMouseMove(container, e),
    )
    canvas.addEventListener("mouseup", (e) => this.handleMouseUp(container, e))
    canvas.addEventListener("dblclick", () => this.clearAllBoxes(container))
  }

  drawRandomBoxesCommand(container) {
    const data = this.instances.get(container)
    // Toggle automatic box drawing
    if (data.autoDrawInterval) {
      clearInterval(data.autoDrawInterval)
      data.autoDrawInterval = null
      console.log("Stopped auto drawing")
    } else {
      console.log("Started auto drawing")
      let max = 253
      data.autoDrawInterval = setInterval(() => {
        if (max) {
          this.drawRandomBox(container)
          max--
        } else {
          if (Math.random() > 0.92) this.drawRandomBox(container)
        }
      }, 25) // Draw every 500ms
    }
  }

  drawRandomBox(container) {
    const data = this.instances.get(container)
    const { canvas } = data

    // Generate random box dimensions within canvas bounds
    const minSize = 10
    const maxSize = 30
    const width = Math.random() * (maxSize - minSize) + minSize
    const height = Math.random() * (maxSize - minSize) + minSize
    const x = Math.random() * (canvas.width - width)
    const y = Math.random() * (canvas.height - height)

    // Add the box to the collection
    data.boxes.push({ x, y, width, height })

    // Redraw all boxes
    this.redrawBoxes(container)

    // Update cell counter
    const hud = container.querySelector(".scrollWebcamHud")
    const cellCounter = hud.querySelector(
      '.scrollWebcamHudElement[data-type="cellCount"]',
    )
    if (cellCounter) {
      cellCounter.textContent = `Cells Counted: ${data.boxes.length}`
    }
  }

  handleMouseDown(container, e) {
    const data = this.instances.get(container)
    const rect = data.canvas.getBoundingClientRect()

    data.startX = e.clientX - rect.left
    data.startY = e.clientY - rect.top
    data.isDrawing = true
  }

  handleMouseMove(container, e) {
    const data = this.instances.get(container)
    if (!data.isDrawing) return

    const rect = data.canvas.getBoundingClientRect()
    const currentX = e.clientX - rect.left
    const currentY = e.clientY - rect.top

    this.clearCanvas(container)
    this.redrawBoxes(container)
    this.drawBox(
      container,
      data.startX,
      data.startY,
      currentX - data.startX,
      currentY - data.startY,
    )
  }

  handleMouseUp(container, e) {
    const data = this.instances.get(container)
    if (!data.isDrawing) return

    const rect = data.canvas.getBoundingClientRect()
    const endX = e.clientX - rect.left
    const endY = e.clientY - rect.top

    const x = Math.min(data.startX, endX)
    const y = Math.min(data.startY, endY)
    const width = Math.abs(endX - data.startX)
    const height = Math.abs(endY - data.startY)

    if (width > 5 && height > 5) {
      data.boxes.push({ x, y, width, height })
      // Update cell counter immediately after adding a box
      const hud = container.querySelector(".scrollWebcamHud")
      const cellCounter = hud.querySelector(
        '.scrollWebcamHudElement[data-type="cellCount"]',
      )
      if (cellCounter) {
        cellCounter.textContent = `Cells Counted: ${data.boxes.length}`
      }
    }

    data.isDrawing = false
    this.redrawBoxes(container)
  }

  clearCanvas(container) {
    const data = this.instances.get(container)
    const { ctx, canvas } = data
    ctx.clearRect(0, 0, canvas.width, canvas.height)
  }

  redrawBoxes(container) {
    const data = this.instances.get(container)

    this.clearCanvas(container)
    data.boxes.forEach((box) => {
      this.drawBox(container, box.x, box.y, box.width, box.height)
    })
  }

  clearAllBoxes(container) {
    const data = this.instances.get(container)
    data.boxes = []
    this.clearCanvas(container)

    // Update cell counter when clearing boxes
    const hud = container.querySelector(".scrollWebcamHud")
    const cellCounter = hud.querySelector(
      '.scrollWebcamHudElement[data-type="cellCount"]',
    )
    if (cellCounter) {
      cellCounter.textContent = "Cells Counted: 0"
    }
  }
}

// Initialize the app when the DOM is loaded
document.addEventListener("DOMContentLoaded", async () => {
  // Verify TensorFlow.js and COCO-SSD are loaded
  if (typeof tf === "undefined") {
    console.error("TensorFlow.js is not loaded")
    return
  }
  if (typeof cocoSsd === "undefined") {
    console.error("COCO-SSD model is not loaded")
    return
  }

  console.log("TensorFlow.js loaded:", typeof tf !== "undefined")
  console.log("COCO-SSD loaded:", typeof cocoSsd !== "undefined")

  const app = new MitoSiteApp()
  window.app = app
  await app.init()
})
